import unittest
import emoji_blender as eb
from collections import Counter


def groups_to_counters(groups):
    return {k: Counter(e.tag for e in v) for k, v in groups.items()}


class TestEmojiFromSVG(unittest.TestCase):

    def test_student_provided_quiz(self):
        """Emoji.from_svg() on example from documentation"""
        svg = """
        <svg xmlns="http://www.w3.org/2000/svg" id="emoji" viewBox="0 0 72 72">
                <g id="color">
                    <circle class="face"/>
                    <rect class="decoration"/>
                </g>
                <g id="line">
                    <path class="mouth"/>
                    <line class="eyes"/>
                </g>
                <g id="skin" />
            </svg>"""
        expected = {
            "face": Counter({"{http://www.w3.org/2000/svg}circle": 1}),
            "decoration": Counter({"{http://www.w3.org/2000/svg}rect": 1}),
            "mouth": Counter({"{http://www.w3.org/2000/svg}path": 1}),
            "eyes": Counter({"{http://www.w3.org/2000/svg}line": 1}),
        }
        groups = eb.Emoji.from_svg(svg).groups
        actual = groups_to_counters(groups)

        self.assertEqual(
            expected.keys(),
            actual.keys(),
            "Keys for the output dictionary do not match.",
        )
        self.assertEqual(
            expected, actual, "Output dict does not have the proper tags."
        )

    def test_from_svg_1F4AC(self):
        """Emoji.from_svg() on 1F4AC"""
        expected = {
            "face": Counter(),
            "decoration": Counter(
                {
                    "{http://www.w3.org/2000/svg}circle": 7,
                    "{http://www.w3.org/2000/svg}path": 2,
                }
            ),
            "mouth": Counter(),
            "eyes": Counter(),
        }
        content_file = open("openmoji/1F4AC.svg", "r")
        content = content_file.read()
        content_file.close()
        groups = eb.Emoji.from_svg(content).groups
        actual = groups_to_counters(groups)

        self.assertEqual(
            expected.keys(),
            actual.keys(),
            "Keys for the output dictionary do not match.",
        )
        self.assertEqual(
            expected, actual, "Output dict does not have the proper tags."
        )

    def test_from_svg_1FAE5(self):
        """Emoji.from_svg() on 1FAE5"""

        expected = {
            "face": Counter({"{http://www.w3.org/2000/svg}circle": 3}),
            "decoration": Counter(),
            "mouth": Counter({"{http://www.w3.org/2000/svg}line": 1}),
            "eyes": Counter({"{http://www.w3.org/2000/svg}path": 2}),
        }
        content_file = open("openmoji/1FAE5.svg", "r")
        content = content_file.read()
        content_file.close()
        groups = eb.Emoji.from_svg(content).groups
        actual = groups_to_counters(groups)

        self.assertEqual(
            expected.keys(),
            actual.keys(),
            "Keys for the output dictionary do not match.",
        )
        self.assertEqual(
            expected, actual, "Output dict does not have the proper tags."
        )

    def test_from_svg_1F480(self):
        """Emoji.from_svg() on 1F480"""

        expected = {
            "face": Counter(
                {
                    "{http://www.w3.org/2000/svg}line": 4,
                    "{http://www.w3.org/2000/svg}path": 3,
                    "{http://www.w3.org/2000/svg}polyline": 1,
                }
            ),
            "decoration": Counter(),
            "mouth": Counter(),
            "eyes": Counter(
                {
                    "{http://www.w3.org/2000/svg}circle": 2,
                    "{http://www.w3.org/2000/svg}path": 2,
                }
            ),
        }
        content_file = open("openmoji/1F480.svg", "r")
        content = content_file.read()
        content_file.close()
        groups = eb.Emoji.from_svg(content).groups
        actual = groups_to_counters(groups)

        self.assertEqual(
            expected.keys(),
            actual.keys(),
            "Keys for the output dictionary do not match.",
        )
        self.assertEqual(
            expected, actual, "Output dict does not have the proper tags."
        )

    def test_from_svg_1F60D(self):
        """Emoji.from_svg() on 1F60D"""

        expected = {
            "face": Counter({"{http://www.w3.org/2000/svg}circle": 2}),
            "decoration": Counter(),
            "mouth": Counter({"{http://www.w3.org/2000/svg}path": 2}),
            "eyes": Counter({"{http://www.w3.org/2000/svg}path": 4}),
        }
        content_file = open("openmoji/1F60D.svg", "r")
        content = content_file.read()
        content_file.close()
        groups = eb.Emoji.from_svg(content).groups
        actual = groups_to_counters(groups)

        self.assertEqual(
            expected.keys(),
            actual.keys(),
            "Keys for the output dictionary do not match.",
        )
        self.assertEqual(
            expected, actual, "Output dict does not have the proper tags."
        )


class TestLoadEmojis(unittest.TestCase):

    def test_load_emojis_full(self):
        """load_emojis() on full emoji set"""
        emojis = eb.EmojiLibrary("openmoji").emojis
        self.assertEqual(len(emojis), 166, "Incorrect no. of emojis loaded.")
        self.assertIn(
            "grinning face", emojis, "Name 'grinning face' not found as key."
        )
        self.assertIn(
            "grinning face with big eyes",
            emojis,
            "Name 'grinning face with big eyes' not found as key.",
        )
